import React, { useEffect, useState } from 'react'

import { UserContext } from 'contexts'

type UserProviderProps = {
  children: React.ReactNode
}

function UserProvider(props: UserProviderProps) {
  const { children } = props
  const [user, setUser] = useState({
    auth: false,
    fetched: false,
    first_name: '',
    last_name: '',
  })

  const getUserInfo = async () => {
    try {
      const response = await fetch('/api/ui/v1/userinfo/')
      const data = await response.json()

      if (response.ok) {
        setUser({
          auth: true,
          fetched: true,
          first_name: data.first_name,
          last_name: data.last_name,
        })
      } else {
        setUser({ auth: false, fetched: true, first_name: '', last_name: '' })
      }
    } catch (error) {
      setUser({ auth: false, fetched: true, first_name: '', last_name: '' })
    }
  }

  const fetchUserInfo = () => {
    if (!user.auth) {
      getUserInfo()
    }
  }

  useEffect(() => {
    if (!user.auth && !user.fetched) {
      getUserInfo()
    }
  }, [])

  const clearUserInfo = () => {
    setUser({ auth: false, fetched: false, first_name: '', last_name: '' })
  }

  return (
    <UserContext.Provider value={{ user, fetchUserInfo, clearUserInfo }}>
      {children}
    </UserContext.Provider>
  )
}

export default UserProvider
